/* AbiSource Program Utilities
 * Copyright (C) 1998 AbiSource, Inc.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 */


#include <memory.h>
#include <malloc.h>

#include "ut_types.h"
#include "ut_assert.h"
#include "ut_debugmsg.h"
#include "ut_string.h"
#include "ps_Font.h"

#define NrElements(a)		((sizeof(a)/sizeof(a[0])))

ps_Font::ps_Font(UT_uint32 ndx)
{
	m_ndx = ndx;
	m_afm_fi = 0;
	m_szFamily = 0;
	m_szFace = 0;
	m_szStyle = 0;
	m_szWeight = 0;
	m_iSize = 0;
}

ps_Font::~ps_Font()
{
	if (m_afm_fi)
	{
		// TODO delete it and everything in it
	}
	if (m_szFamily)
		free(m_szFamily);
	if (m_szFace)
		free(m_szFace);
	if (m_szStyle)
		free(m_szStyle);
}

UT_Bool ps_Font::loadFont(const char * filename)
{
	char szFullPath[1024];
	// TODO don't use a hard coded path
	// but until then...check some known places
	sprintf(szFullPath,"/usr/share/ghostscript/fonts/%s", filename);
	FILE * fp;
	fp = fopen(szFullPath,"r");
	if (!fp)
	{
		// try a second possibility
		sprintf(szFullPath,"/usr/lib/ghostscript/fonts/%s", filename);
		fp = fopen(szFullPath,"r");
		if(!fp)
		{
			// try a third possibility
			sprintf(szFullPath,"/usr/local/ghostscript/fonts/%s", filename);
			fp = fopen(szFullPath,"r");
			// Darn...can't find the fonts
			if(!fp)
				return UT_FALSE;
		}
	}

	int result = parseFile(fp,&m_afm_fi,P_GW);
	if (result != ok)
	{
		UT_DEBUGMSG(("ps_Font::loadFont: [file %s] [error %d]\n",filename,result));
		return UT_FALSE;
	}
	UT_ASSERT(m_afm_fi);
	UT_ASSERT(m_afm_fi->gfi);
	UT_DEBUGMSG(("    [font %s][family %s][weight %s][italic %f][fixed %d]\n",
				 m_afm_fi->gfi->fontName,m_afm_fi->gfi->familyName,m_afm_fi->gfi->weight,
				 m_afm_fi->gfi->italicAngle,m_afm_fi->gfi->isFixedPitch));

	return UT_TRUE;
}
UT_Bool ps_Font::setFont(const char * szFamily, const char * szFace, 
							const char * szStyle, const char * szWeight, 
							UT_uint32 iSize)
{
	UT_cloneString(m_szFamily,szFamily);
	UT_cloneString(m_szFace,szFace);
	UT_cloneString(m_szStyle,szStyle);
	UT_cloneString(m_szWeight,szWeight);
	m_iSize = iSize;
	
	return UT_TRUE;
}

UT_Bool ps_Font::matchesFont(const char * szFamily, const char * szFace, 
								const char * szStyle, const char * szWeight, UT_uint32 iSize)
{
	// return true if the given args match this font.

	return (   (UT_stricmp(m_szFamily,szFamily) == 0)
			&& (UT_stricmp(m_szFace,szFace) == 0)
			&& (UT_stricmp(m_szStyle,szStyle) == 0)
			&& (UT_stricmp(m_szWeight,szWeight) == 0)
			&& (m_iSize == iSize));
}

