/* AbiSource Program Utilities
 * Copyright (C) 1998 AbiSource, Inc.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA.
 */
 
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include "ut_hash.h"
#include "ut_debugmsg.h"
#include "ut_assert.h"
#include "ut_string.h"
#include "ps_Fontmap.h"

ps_Fontmap::ps_Fontmap()
{
	m_pFontHash = NULL;
	m_pAliasHash = NULL;
	m_pCSSHash = NULL;
}

ps_Fontmap::~ps_Fontmap()
{
	if(m_pFontHash)
		delete m_pFontHash;
	if(m_pAliasHash)
		delete m_pAliasHash;
	if(m_pCSSHash)
		delete m_pCSSHash;
}

UT_Bool ps_Fontmap::createFontHash()
{
	m_pFontHash = new UT_HashTable(10);
	m_pAliasHash = new UT_HashTable(10);
	m_pCSSHash = new UT_HashTable(10);

	UT_Bool result = parseFontmap();
	if (!result)
		return UT_FALSE;

	result = addCSSAliases();
	if (!result)
		return UT_FALSE;

	return result;
}

UT_Bool ps_Fontmap::addCSSAliases()
{
	// TODO make this into a parseable file somewhere...
	// add some default aliases...
	// These are aliases to aliases
	m_pCSSHash->addEntry("times-normal-normal", "Times-Roman", NULL);
	m_pCSSHash->addEntry("times-normal-bold", "Times-Bold", NULL);
	m_pCSSHash->addEntry("times-italic-normal", "Times-Italic", NULL);
	m_pCSSHash->addEntry("times-italic-bold", "Times-BoldItalic", NULL);
	m_pCSSHash->addEntry("Times New Roman-normal-normal", "Times-Roman", NULL);
	m_pCSSHash->addEntry("Times New Roman-normal-bold", "Times-Bold", NULL);
	m_pCSSHash->addEntry("Times New Roman-italic-normal", "Times-Italic", NULL);
	m_pCSSHash->addEntry("Times New Roman-italic-bold", "Times-BoldItalic", NULL);

	m_pCSSHash->addEntry("courier-normal-normal", "Courier", NULL);
	m_pCSSHash->addEntry("courier-normal-bold", "Courier-Bold", NULL);
	m_pCSSHash->addEntry("courier-italic-normal", "Courier-Oblique", NULL);
	m_pCSSHash->addEntry("courier-italic-bold", "Courier-BoldOblique", NULL);

	m_pCSSHash->addEntry("helvetica-normal-normal", "Helvetica", NULL);
	m_pCSSHash->addEntry("helvetica-normal-bold", "Helvetica-Bold", NULL);
	m_pCSSHash->addEntry("helvetica-italic-normal", "Helvetica-Oblique", NULL);
	m_pCSSHash->addEntry("helvetica-italic-bold", "Helvetica-BoldOblique", NULL);
	m_pCSSHash->addEntry("helvetica-oblique-normal", "Helvetica-Oblique", NULL);
	m_pCSSHash->addEntry("helvetica-oblique-bold", "Helvetica-BoldOblique", NULL);

	m_pCSSHash->addEntry("arial-normal-normal", "Helvetica", NULL);
	m_pCSSHash->addEntry("arial-normal-bold", "Helvetica-Bold", NULL);
	m_pCSSHash->addEntry("arial-italic-normal", "Helvetica-Oblique", NULL);
	m_pCSSHash->addEntry("arial-italic-bold", "Helvetica-BoldOblique", NULL);
	m_pCSSHash->addEntry("arial-oblique-normal", "Helvetica-Oblique", NULL);
	m_pCSSHash->addEntry("arial-oblique-bold", "Helvetica-BoldOblique", NULL);

	m_pCSSHash->addEntry("palatino-normal-normal", "Palatino-Roman", NULL);
	m_pCSSHash->addEntry("palatino-normal-bold", "Palatino-Bold", NULL);
	m_pCSSHash->addEntry("palatino-italic-normal", "Palatino-Italic", NULL);
	m_pCSSHash->addEntry("palatino-italic-bold", "Palatino-BoldItalic", NULL);

	m_pCSSHash->addEntry("new century schoolbook-normal-normal", "NewCenturySchlbk-Roman", NULL);
	m_pCSSHash->addEntry("new century schoolbook-normal-bold", "NewCenturySchlbk-Bold", NULL);
	m_pCSSHash->addEntry("new century schoolbook-italic-normal", "NewCenturySchlbk-Italic", NULL);
	m_pCSSHash->addEntry("new century schoolbook-italic-bold", "NewCenturySchlbk-BoldItalic", NULL);

	//	These are not aliases, but actual font names...
	m_pCSSHash->addEntry("charter-normal-normal", "Charter-Roman", NULL);
	m_pCSSHash->addEntry("charter-normal-bold", "Charter-Bold", NULL);
	m_pCSSHash->addEntry("charter-italic-normal", "Charter-Italic", NULL);
	m_pCSSHash->addEntry("charter-italic-bold", "Charter-BoldItalic", NULL);

	m_pCSSHash->addEntry("utopia-normal-normal", "Utopia-Regular", NULL);
	m_pCSSHash->addEntry("utopia-normal-bold", "Utopia-Bold", NULL);
	m_pCSSHash->addEntry("utopia-italic-normal", "Utopia-Italic", NULL);
	m_pCSSHash->addEntry("utopia-italic-bold", "Utopia-BoldItalic", NULL);

	return UT_TRUE;
}
/**	mapFontName makes checks in three hashtables. First, it takes the CSS properties
	and looks up an alias to a font. Second, it uses this alias to lookup the font
	name. Third, it uses this font name to lookup an *.afm file to use.

	We could have the case where the CSS name maps to a font name, bypassing aliases.
	So step two could be skipped.

*/
UT_Bool ps_Fontmap::mapFontName(const char * szFamily, const char * szFace, 
							const char * szStyle, const char * szWeight,
							char * pBuffer, UT_uint32 bufferLength)
{
	UT_HashTable::UT_HashEntry *fontCSSEntry = NULL;
	UT_HashTable::UT_HashEntry *fontAliasEntry = NULL;
	UT_HashTable::UT_HashEntry *fontEntry = NULL;

	// Build a font lookup entry from the CSS properties available
	char szFontDesc[256];
	sprintf(szFontDesc, "%s-%s-%s", szFamily, szStyle, szWeight);

	// STEP 1 - lookup CSS font name
	fontCSSEntry = m_pCSSHash->findEntry(szFontDesc);

	// Two things can happen here...either maps to an alias or straight to a font.
	if(!fontCSSEntry)
	{
		// if no CSS entry...try default of a times-same style-same weight
		char szDefault[256];
		sprintf(szDefault, "times-%s-%s", szStyle, szWeight);
		fontCSSEntry = m_pCSSHash->findEntry(szDefault);
		// Sorry, can't even find a default font
		if(!fontCSSEntry)
			return UT_FALSE;
	}

	// STEP 2 - lookup alias to font name...this may fail, it is OK
	// we will catch it in Step 3
	fontAliasEntry = m_pAliasHash->findEntry(fontCSSEntry->pszRight);

	// STEP 3 - get entry to file name
	if(fontAliasEntry)
		fontEntry = m_pFontHash->findEntry(fontAliasEntry->pszRight);
	else
		fontEntry = m_pFontHash->findEntry(fontCSSEntry->pszRight);

	// If nothing found, get a default mapping that should be present...
	if(!fontEntry)
	{
		fontEntry = m_pFontHash->findEntry("NimbusRomNo9L-Regu");
		// Too bad...default font not found
		if(!fontEntry)
			return UT_FALSE;
	}

	char * buf = new char [bufferLength + 1];
	UT_ASSERT(buf);
	
	sprintf(buf, "%s", fontEntry->pszRight);

	if(bufferLength < strlen(buf))
		return UT_FALSE;
	strcpy(pBuffer, buf);

	if (buf)
		delete buf;
	
	return UT_TRUE;
}


UT_Bool ps_Fontmap::parseFontmap()
{
	// TODO This looks in same directory as the binary...we should
	// TODO have a preference setting for this.
	ifstream ifs ;
	ifs.open("/usr/local/ghostscript/fonts/Fontmap");
	if(!ifs)
		ifs.open("/usr/share/ghostscript/fonts/Fontmap");
	if(!ifs)
		ifs.open("/usr/lib/ghostscript/fonts/Fontmap");
	if(!ifs)
		ifs.open("/etc/gs.Fontmap");
	if(!ifs)
		ifs.open("Fontmap");
	if(!ifs)
	{
		UT_DEBUGMSG(("I cannot find a Fontmap file!"));
		return UT_FALSE;
	}

	char fline[256];
	while ( ifs.getline(fline, 256) )
	{
		// if not a comment and not a blank line
		if(fline[0] != '%' && fline[0] != '\0')
		{
			char *key = strtok(fline, " \t");
			char *value = strtok(NULL, " \t");
			char *newkey;
			char *newvalue;

			if( !strncmp( value, "(", 1 ))	// if font filename
			{
				newkey = strtok(key, "/");
				newvalue = strtok(value, "().");
				strcat(newvalue, ".afm");
				m_pFontHash->addEntry(newkey, newvalue, NULL);
			}
			else	// it is an alias
			{
				newvalue = strtok(value, "/");
				newkey = strtok(key, "/");
				m_pAliasHash->addEntry(newkey, newvalue, NULL);
			}
		}
	}

	ifs.close();
	return UT_TRUE;
}

